==> analytics.js-integration-marketo-v2-3.0.2.tgz: package/lib/index.js <==
'use strict';

/**
 * Module dependencies.
 */

var fmt = require('@segment/fmt');
var integration = require('@segment/analytics.js-integration');
var is = require('is');
var jsonp = require('jsonp');
var url = require('component-url');
var when = require('do-when');
var each = require('@ndhoule/each');

/**
 * Expose `Marketo` integration.
 */

var Marketo = module.exports = integration('Marketo V2')
  .assumesPageview()
  .global('Munchkin')
  .option('host', 'https://api.segment.io')
  .option('accountId', '')
  .option('projectId', '')
  .option('traits', [])
  .tag('<script src="//munchkin.marketo.net/munchkin.js">');

/**
 * Initialize.
 *
 * https://app-q.marketo.com/#MS0A1
 *
 * @api public
 */

Marketo.prototype.initialize = function() {
  var self = this;
  this.load(function() {
    window.Munchkin.init(self.options.accountId, {
      asyncOnly: true
    });
    // marketo integration actually loads a marketo snippet
    // and the snippet loads the real marketo, this is required
    // because there's a race between `window.mktoMunchkinFunction = sinon.spy()`
    // and marketo's real javascript which overrides `window.mktoMunchkinFunction`
    // and deletes the spy.
    when(self.loaded, self.ready);
  });
};

/**
 * Loaded?
 *
 * @api public
 * @return {boolean}
 */

Marketo.prototype.loaded = function() {
  return is.object(window.Munchkin) && !!window.MunchkinTracker;
};

/**
 * Page.
 *
 * @api public
 * @param {Page}
 */

Marketo.prototype.page = function(page) {
  // Remove once V1 is completely deprecated.
  if (!page.enabled('Marketo')) return;

  var properties = page.properties();
  var parsed = url.parse(properties.url);
  window.mktoMunchkinFunction('visitWebPage', {
    url: properties.url,
    params: parsed.query
  });
};

/**
 * Identify.
 *
 * @api public
 * @param {Identify} identify
 */

Marketo.prototype.identify = function(identify) {
  // Remove once V1 is completely deprecated.
  if (!identify.enabled('Marketo')) {
    return;
  }

  var settings = this.options;

  // we _must_ have an email
  var email = identify.email();
  if (!email) return;

  // setting standard traits
  var traitsToSendMarketo = {};
  var traits = identify.traits();
  var company = identify.proxy('traits.company');
  var address = identify.address();

  traitsToSendMarketo.email = email;
  if (identify.created()) traitsToSendMarketo.createdAt = identify.created();
  if (identify.birthday()) traitsToSendMarketo.dateOfBirth = identify.birthday();
  if (identify.firstName()) traitsToSendMarketo.firstName = identify.firstName();
  if (identify.gender()) traitsToSendMarketo.gender = identify.gender();
  if (identify.lastName()) traitsToSendMarketo.lastName = identify.lastName();
  if (identify.phone()) traitsToSendMarketo.phone = identify.phone();
  if (identify.userId()) traitsToSendMarketo.userId = identify.userId();
  if (identify.anonymousId()) traitsToSendMarketo.anonymousId = identify.anonymousId();

  if (address) {
    if (is.object(address)) {
      traitsToSendMarketo.city = address.city;
      traitsToSendMarketo.country = address.country;
      traitsToSendMarketo.postalCode = address.zip || address.postalCode;
      traitsToSendMarketo.state = address.state;
    } else {
      traitsToSendMarketo.address = address;
    }
  }

  if (company) {
    if (is.object(company)) {
      traitsToSendMarketo.company = company.name;
    } else {
      traitsToSendMarketo.company = company;
    }
  }

  each(function(trait) {
    var segmentTrait = trait.value.segmentTrait;
    var marketoField = trait.value.marketoFieldName;
    if (traits[segmentTrait]) {
      traitsToSendMarketo[marketoField] = traits[segmentTrait];
    }
  }, settings.traits);

  // associate the lead on the client-side so that
  // we can track to the same user
  this.requestHash(traitsToSendMarketo.email, function(err, hash) {
    var marketoFn = window.mktoMunchkinFunction;
    if (marketoFn) marketoFn('associateLead', traitsToSendMarketo, hash);
  });
};

/**
 * Generate the URL to the Segment endpoint that hashes Marketo emails.
 *
 * @api private
 * @param {string} email
 * @return {string}
 */

Marketo.prototype.emailHashUrl = function(email) {
  var host = this.options.host;
  var projectId = this.options.projectId;
  return fmt('%s/integrations/marketo/v1/%s/%s/hash', host, projectId, email);
};

/**
 * Marketo requires that users' requests come with a hashed version of their
 * email address. It's a hash that can't be done on the client-side for some
 * reason.
 *
 * Man, it'd have been great if someone had documented this in the first place.
 *
 * See https://github.com/segmentio/marketo-hash-app for more details.
 *
 * TODO: Improve this documentation
 *
 * @api private
 * @param {string} email
 * @param {Function} callback
 */

Marketo.prototype.requestHash = function(email, callback) {
  var url = this.emailHashUrl(email);
  jsonp(url, callback);
};

==> analytics.js-integration-marketo-v2-3.0.3.tgz: package/lib/index.js <==
'use strict';

/**
 * Module dependencies.
 */

var fmt = require('@segment/fmt');
var integration = require('@segment/analytics.js-integration');
var is = require('is');
var jsonp = require('jsonp');
var url = require('component-url');
var when = require('do-when');
var each = require('@ndhoule/each');

/**
 * Expose `Marketo` integration.
 */

var Marketo = module.exports = integration('Marketo V2')
  .assumesPageview()
  .global('Munchkin')
  .option('host', 'https://api.segment.io')
  .option('accountId', '')
  .option('projectId', '')
  .option('traits', [])
  .tag('<script src="//munchkin.marketo.net/munchkin.js">');

/**
 * Initialize.
 *
 * https://app-q.marketo.com/#MS0A1
 *
 * @api public
 */

Marketo.prototype.initialize = function() {
  var self = this;
  this.load(function() {
    window.Munchkin.init(self.options.accountId, {
      asyncOnly: true
    });
    // marketo integration actually loads a marketo snippet
    // and the snippet loads the real marketo, this is required
    // because there's a race between `window.mktoMunchkinFunction = sinon.spy()`
    // and marketo's real javascript which overrides `window.mktoMunchkinFunction`
    // and deletes the spy.
    when(self.loaded, self.ready);
  });
};

/**
 * Loaded?
 *
 * @api public
 * @return {boolean}
 */

Marketo.prototype.loaded = function() {
  return is.object(window.Munchkin) && !!window.MunchkinTracker;
};

/**
 * Page.
 *
 * @api public
 * @param {Page}
 */

Marketo.prototype.page = function(page) {
  // Remove once V1 is completely deprecated.
  if (!page.enabled('Marketo')) return;

  var properties = page.properties();
  var parsed = url.parse(properties.url);
  window.mktoMunchkinFunction('visitWebPage', {
    url: properties.url,
    params: parsed.query
  });
};

/**
 * Identify.
 *
 * @api public
 * @param {Identify} identify
 */

Marketo.prototype.identify = function(identify) {
  // Remove once V1 is completely deprecated.
  if (!identify.enabled('Marketo')) {
    return;
  }

  var settings = this.options;

  // we _must_ have an email
  var email = identify.email();
  if (!email) return;

  // setting standard traits
  var traitsToSendMarketo = {};
  var traits = identify.traits();
  var company = identify.proxy('traits.company');
  var address = identify.address();

  traitsToSendMarketo.email = email;
  if (identify.created()) traitsToSendMarketo.createdAt = identify.created();
  if (identify.birthday()) traitsToSendMarketo.dateOfBirth = identify.birthday();
  if (identify.firstName()) traitsToSendMarketo.firstName = identify.firstName();
  if (identify.gender()) traitsToSendMarketo.gender = identify.gender();
  if (identify.lastName()) traitsToSendMarketo.lastName = identify.lastName();
  if (identify.phone()) traitsToSendMarketo.phone = identify.phone();
  if (identify.userId()) traitsToSendMarketo.userId = identify.userId();
  if (identify.anonymousId()) traitsToSendMarketo.anonymousId = identify.anonymousId();

  if (address) {
    if (is.object(address)) {
      traitsToSendMarketo.city = address.city;
      traitsToSendMarketo.country = address.country;
      traitsToSendMarketo.postalCode = address.zip || address.postalCode;
      traitsToSendMarketo.state = address.state;
    } else {
      traitsToSendMarketo.address = address;
    }
  }

  if (company) {
    if (is.object(company)) {
      traitsToSendMarketo.company = company.name;
    } else {
      traitsToSendMarketo.company = company;
    }
  }

  each(function(trait) {
    var segmentTrait = trait.value.segmentTrait;
    var marketoField = trait.value.marketoFieldName;
    if (traits[segmentTrait]) {
      traitsToSendMarketo[marketoField] = traits[segmentTrait];
    }
  }, settings.traits);

  // associate the lead on the client-side so that
  // we can track to the same user
  this.requestHash(traitsToSendMarketo.email, function(err, hash) {
    var marketoFn = window.mktoMunchkinFunction;
    if (marketoFn) marketoFn('associateLead', traitsToSendMarketo, hash);
  });
};

/**
 * Generate the URL to the Segment endpoint that hashes Marketo emails.
 *
 * @api private
 * @param {string} email
 * @return {string}
 */

Marketo.prototype.emailHashUrl = function(email) {
  var host = this.options.host;
  var projectId = this.options.projectId;
  return fmt('%s/integrations/marketo/v1/%s/%s/hash-v2', host, projectId, email);
};

/**
 * Marketo requires that users' requests come with a hashed version of their
 * email address. It's a hash that can't be done on the client-side for some
 * reason.
 *
 * Man, it'd have been great if someone had documented this in the first place.
 *
 * See https://github.com/segmentio/marketo-hash-app for more details.
 *
 * TODO: Improve this documentation
 *
 * @api private
 * @param {string} email
 * @param {Function} callback
 */

Marketo.prototype.requestHash = function(email, callback) {
  var url = this.emailHashUrl(email);
  jsonp(url, callback);
};
